/*!
  \class ConfigManager configmanager.h
  \brief This class manages the config file reader and the configuration dialog

  The ConfigManager class creates and manages the configuration file parser and
  also contains the configuration dialog, though this may soon change.
*/

#include "configmanager.h"
#include "configdlg.h"

// #include "appconf.h"

/*!
  This is the constructor, currently it creates a new FileConfig object
  and a ConfigDisplay object.  I am considering moving the ConfigDisplay object
  elsewhere.  But for now it remains here.
*/

ConfigManager::ConfigManager( QObject *parent ) 
  : QObject( parent )
{
  configParser = new FileConfig( "mysqltutor.conf", false, false );
  configDialog = new ConfigDialog();

  QObject::connect( configDialog, SIGNAL(applyButtonPressed()), this, SLOT(applyPressed()) );
}

/*!
  This is the destructor.  Currently I have it deleting the FileConfig.
  I do this to ensure that the FileConfig's dtor is called in case something
  drastic happens to this object.  This is so all current configuration values
  get written to the file.
*/

ConfigManager::~ConfigManager()
{
  delete configParser;
}

/*!
  This member shows the configuration dialog.
  Also sets the values in each edit field and/or check box.  There
  has got to be a better way to do this.  It is a complete mess right now.
*/

void ConfigManager::showDialog()
{
  configDialog->show();

  setHostEdit( getHost() );
  setUserEdit( getUser() );
  setPassEdit( getPass() );
  setDBEdit( getDBase() );
  setConButton( getAutoConnect() );
  setLastLessButton( getLessonTrack() );
  setGeoKeepButton( getKeepGeom() );
  setSplitKeepButton( getKeepSplit() );
  setLessonDirEdit( getLessonDir() );
}

/*!
  This member returns weather or not autoconnect on startup is turned on.

  True = turned on

  False = turned off
*/

bool ConfigManager::getAutoConnect()
{
  configParser->setCurrentPath("startup");
  if (strcmp( configParser->readEntry("autoconnect", "false"), "true" ) == 0 )
    return true;
  else
    return false;
}

/*!
  This member returns weather or not lesson tracking is turned on.

  True = turned on

  False = turned off
*/

bool ConfigManager::getLessonTrack()
{
  configParser->setCurrentPath("startup");
  if (strcmp( configParser->readEntry("startonlast", "false"), "true" ) == 0 )
    return true;
  else
    return false;
}

/*!
  This member returns weather or not the last geometry memory is turned on.

  True = turned on

  False = turned off
*/

bool ConfigManager::getKeepGeom()
{
  configParser->setCurrentPath("startup");
  if (strcmp( configParser->readEntry("keepgeometry", "false"), "true" ) == 0 )
    return true;
  else
    return false;
}

/*!
  This member returns weather or not the last splitter position memeory is turned on.

  True = turned on

  False = turned off
*/

bool ConfigManager::getKeepSplit()
{
  configParser->setCurrentPath("startup");
  if (strcmp( configParser->readEntry("keepsplitter", "false"), "true" ) == 0 )
    return true;
  else
    return false;
}

/*!
  This member returns weather or not to use big icons in the tool bar.

  True = use big icons

  False = use small icons
*/

bool ConfigManager::getIconStatus()
{
  configParser->setCurrentPath("appearance");
  if (strcmp( configParser->readEntry("iconbig", "false"), "true" ) == 0 )
    return true;
  else
    return false;
}

/*!
  \fn const char* ConfigManager::getHost()
  This member returns the value of the database's host.
*/

/*!
  \fn const char* ConfigManager::getUser()
  This member returns the value of the database username.
*/

/*!
  \fn const char* ConfigManager::getPass()
  This member returns the value of the database password.
*/

/*!
  \fn const char* ConfigManager::getDBase()
  This member returns the value of the database to use.
*/

/*!
  \fn const char* ConfigManager::getLessonDir()
  This member returns the value of the directory the lessons are stored in.
*/

/*!
  \fn const int ConfigManager::getLastLesson() const
  This member returns the value of the last lesson, used if lesson tracking
  is turned on.
*/

/*!
  This member retrives a pointer to the host QLineEdit and sets
  its text to the string passed as data.
*/

void ConfigManager::setHostEdit( const char *data )
{
  QLineEdit *temp =  configDialog->getHostPtr();
  temp->setText(data);
}

/*!
  This member retrieves a pointer to the user QLineEdit and sets
  its text to the string passed as data.
*/

void ConfigManager::setUserEdit( const char *data )
{
  QLineEdit *temp = configDialog->getUserPtr();
  temp->setText(data);
}

/*!
  This member retrieves a pointer to the password QLineEdit and sets
  its text to the string passed as data.
*/

void ConfigManager::setPassEdit( const char *data )
{
  QLineEdit *temp = configDialog->getPassPtr();
  temp->setText(data);
}

/*!
  This member retrieves a pointer to the database QLineEdit and sets
  its text to the string passed as data.
*/

void ConfigManager::setDBEdit( const char *data )
{
  QLineEdit *temp = configDialog->getDBPtr();
  temp->setText(data);
}

/*!
  This member retrieves a pointer to the startup autoconnect QRadioButton
  and either checks it or unchecks the "Yes" button according to the value
  it is passed.

  True = "Yes" checked, "No" unchecked

  False = "Yes" unchecked, "No" checked
*/

void ConfigManager::setConButton( bool set )
{
  QRadioButton *temp = configDialog->getConYesPtr();
  temp->setChecked( set );
}

/*!
  This member retrieves a pointer to the startup lastlesson QRadioButton
  and either checks it or unchecks the "Yes" button according to the value
  it is passed.

  True = "Yes" checked, "No" unchecked

  False = "Yes" unchecked, "No" checked
*/

void ConfigManager::setLastLessButton( bool set )
{
  QRadioButton *temp = configDialog->getLessYesPtr();
  temp->setChecked( set );
}

/*!
  This member retrieves a pointer to the startup keepgeometry QRadioButton
  and either checks it or unchecks the "Yes" button according to the value
  it is passed.

  True = "Yes" checked, "No" unchecked

  False = "Yes" unchecked, "No" checked
*/

void ConfigManager::setGeoKeepButton( bool set )
{
  QRadioButton *temp = configDialog->getGeoYesPtr();
  temp->setChecked( set );
}

/*!
  This member retrieves a pointer to the startup keepsplitter QRadioButton
  and either checks it or unchecks the "Yes" button according to the value 
  it is passe.

  True = "Yes" checked, "No" unchecked

  False = "Yes" unchecked, "No" checked
*/

void ConfigManager::setSplitKeepButton( bool set )
{
  QRadioButton *temp = configDialog->getSplitYesPtr();
  temp->setChecked( set );
}

/*!
  This member retrieves a pointer to the startup lesson directory QLineEdit
  and sets its text to the value of lessonDirectory
*/

void ConfigManager::setLessonDirEdit( const char *data )
{
  QLineEdit *temp = configDialog->getDirEdPtr();
  temp->setText( data );
}

/*!
  This slot takes action when the "Apply" or "Ok" button is presse on
  the config dialog.  It simply writes all the new values back to the config
  file and then flushes it.  I may implement a scheme in which it only writes
  values that have changed in the future, to be more efficient.
*/

void ConfigManager::applyPressed()
{

  // set host entry
  QLineEdit *temp = configDialog->getHostPtr();
  configParser->setCurrentPath("database"); 
  configParser->writeEntry("host", temp->text() );

  // set user entry
  temp = configDialog->getUserPtr();
  configParser->writeEntry("user", temp->text() );

  // set password entry
  temp = configDialog->getPassPtr();
  configParser->writeEntry("pass", temp->text() );

  // set database entry
  temp = configDialog->getDBPtr();
  configParser->writeEntry("database", temp->text() );

  // write autoconnect
  configParser->setCurrentPath("startup");
  QRadioButton *tempB = configDialog->getConYesPtr();
  if( tempB->isChecked() )
    configParser->writeEntry("autoconnect", "true" );
  else
    configParser->writeEntry("autoconnect", "false" );

  // write start on last lesson
  tempB = configDialog->getLessYesPtr();
  if( tempB->isChecked() )
    configParser->writeEntry("startonlast", "true" );
  else
    configParser->writeEntry("startonlast", "false" );

  // write keep geometry
  tempB = configDialog->getGeoYesPtr();
  if( tempB->isChecked() )
    configParser->writeEntry("keepgeometry", "true" );
  else
    configParser->writeEntry("keepgeometry", "false" );

  // write keep splitter
  tempB = configDialog->getSplitYesPtr();
  if( tempB->isChecked() )
    configParser->writeEntry("keepsplitter", "true" );
  else
    configParser->writeEntry("keepsplitter", "false" );

  // write lesson directory
  temp = configDialog->getDirEdPtr();
  configParser->setCurrentPath("lesson");
  configParser->writeEntry("lessondir", temp->text() );

  configParser->flush();
}
