/*!
  \class TutorMainWidget mainwidget.h
  \brief This widget contains the lesson, example, and table wigets.

  This widget contains the lesson, example, and table widgets separated by a QSplitter.
  I have abstracted things in this fashion so that I may change the lesson editor, example
  editor, and table display separately and with minimal hassle.
*/

#include "mainwidget.h"

#include "lessondisplay.h"
#include "expleditor.h"
#include "sqltable.h"

#include <qsplitter.h>
#include <qfile.h>
#include <qstring.h>
#include <qtstream.h>

#include <iostream.h>

/*!
  This iss the constructor.  Currently I pass it a pointer to the ConfigManager so that
  I may retrieve the name of the lesson directory.  I do it this way, instead of using 
  a string internal to the class so that I can minimize the amount of things that need
  to be done when the configuration file changes while the program is running.  This way, 
  I will always have current values.
*/

TutorMainWidget::TutorMainWidget( QWidget *parent, const char *name, ConfigManager *fp )
  : QWidget( parent, name ), configParser(fp)
{
  vsplitter = new QSplitter( QSplitter::Vertical, this, "vertical splitter" );

  hsplitter = new QSplitter( QSplitter::Horizontal, vsplitter, "horizontal splitter" );

  lessonDisp = new LessonDisplay( hsplitter, "lesson display" );
  explEdit = new ExampleEditor( hsplitter, "example editor" );

  resultsTable = new SQLTable( 5, 5, vsplitter, "results table" );
  resultsTable->setRows( 2 );
  resultsTable->setCols( 2 );

  lessonDisp->setMinimumSize( 100, 200 );
  explEdit->setMinimumSize( 100, 200 );

  hsplitter->setStyle( WindowsStyle );
  vsplitter->setStyle( WindowsStyle );

  hsplitter->show();
}

/*!
  This is the destructor.  Currently it does nothing.
*/

TutorMainWidget::~TutorMainWidget(){}

/*!
  This is geometry managment for the QSplitter.  I may change
  this widget to use a layout instead, if this fails to work out well.
*/

void TutorMainWidget::resizeEvent( QResizeEvent * )
{
  vsplitter->setGeometry( 0, 0, width(), height() );
}

/*!
  This member returns a pointer to the SQLTable.  It is used in 
  the main window while running queries from the editor.
*/

SQLTable * TutorMainWidget::getTableHandle()
{
  return resultsTable;
}

/*!
  This member returns a pointer to the ExampleEditor.  It is used
  in the main window to clear and reload it.
*/

QMultiLineEdit * TutorMainWidget::getExampleHandle()
{
  return explEdit;
}

/*!
  This member loats the lesson specified by the int arg.
  I plan on moving lesson and example file handling into its
  own class in the near future.  This works for now, so this 
  is how its gonna be until that time.
*/

void TutorMainWidget::loadLessonText( int lesson )
{
  QString lessonfile, explfile, filestring;
  
  lessonfile.setNum(lesson) += ".les";
  lessonfile.prepend( "lesson" );
  lessonfile.prepend( configParser->getLessonDir() );
  
  explfile.setNum(lesson) += ".exp";
  explfile.prepend( "lesson" );
  explfile.prepend( configParser->getLessonDir() );
  
  QFile currentlesson( lessonfile );
  QFile currentexample( explfile );
    
  if( currentlesson.exists() )
    {
      currentlesson.open( IO_ReadOnly );
      currentlesson.at( 0 );
      
      QTextStream t( &currentlesson );
      
      // get title for label
      filestring = t.readLine();
      if( !strcmp(filestring.left(1), "!") )
 	lessonLabel = filestring.right( filestring.size() - 2 );
      filestring = "";
      
      while ( !t.eof() )
 	filestring += t.readLine() + '\n';
      currentlesson.close();
      currentLesson = lesson;
    }
  else
    return;
  lessonDisp->setText( filestring );
  
  emit(lessonChanged());
  
  filestring = "";
  
  if( currentexample.exists() )
    {
      currentexample.open( IO_ReadOnly );
      currentexample.at( 0 );
      
      QTextStream t( &currentexample );
      while( !t.eof() )
  	filestring += t.readLine() + '\n';
      currentexample.close();
    }
  else
    return;
  explEdit->setText( filestring );
}

/*!
  This member loads the next lesson and example.
*/

void TutorMainWidget::fwdlesson()
{
  // currentLesson++;
  loadLessonText( currentLesson + 1 );
}

/*!
  This member loads the previous lesson and example.
*/

void TutorMainWidget::backlesson()
{
  // currentLesson--;
  loadLessonText( currentLesson - 1 );
}

/*!
  This member loads lesson and example 0.
*/

void TutorMainWidget::toplesson()
{
  currentLesson = 0;
  loadLessonText( currentLesson );
}

/*!
  This member reloads the current lesson and example.
  It should really only reload the example.  This will be 
  fixed when I further abstract the lesson/example file
  handler.
*/

void TutorMainWidget::reloadexample()
{
  loadLessonText( currentLesson );
}

/*!
  \fn void TutorMainWidget::lessonChanged() 
  This signal is used to signal the main window when the lesson changes
  so that it may set the status bar with the lessons title.
*/
