/*!
  \class SQLTable sqltable.h
  \brief This widget is the table to display the query results.

  This widget is the table to display the query results.  It inherits
  QTableView.  It automatically sizes to the largest string in the table
  and automatically paints the column headings in blue.  I need to reimplement
  the sizing determination method to optimize redraw.  I plan to loop through
  the results set twice and get the max size before displaying the table. Currently
  I loop through only once and set the table width to the current string width if
  it was larger than the previous strings width.  This makes for a visible redraw
  which I desire to not have.
*/

#include "sqltable.h"

#include <iostream.h>

#include <qpainter.h>
#include <qfontmet.h>

/*!
  This is the constructor.  It sets up the number of rows and columns, specified as args.
  It also sets the tables flags and frame style.  Here it also allocates the restuls string
  on the heap.
*/

SQLTable::SQLTable( int numCols, int numRows, QWidget *parent, const char *name )
  : QTableView( parent, name )
{
  rowNum = numRows;
  colNum = numCols;
  setTableFlags( Tbl_autoScrollBars |
		 Tbl_clipCellPainting |
		 Tbl_smoothScrolling );
  setFrameStyle( QFrame::Box | QFrame::Raised );
  contents = new QString[numRows * numCols];
}

/*!
  This is the destructor.  I do not delete contents here because I believe it to 
  be deleted after each table display.  The reason I think this way is because I
  get core dumps from free() if I delete it.  So it must be fine the way it is.
*/

SQLTable::~SQLTable()
{
}

/*!
  This member sets the data in row, col to the string in data.
*/

void SQLTable::setCell(int row, int col, const char *data )
{
  contents[indexOf( row, col )] = data;          // contents lookup and assign
  
  QFontMetrics fm( font() );
  if( fm.width( contents[indexOf( row, col )] ) > cellWidth() )
    setCellWidth( fm.width( contents[indexOf( row, col )] + 100 ) );
  
  int newHeight = 0;
  QString testString( contents[indexOf(row,col)] );
  
  newHeight = ( ((fm.height() * (testString.contains('\n') + 1)) + 10) );
  
  if( newHeight > cellHeight() )
    setCellHeight( newHeight );
  
  updateCell( row, col );                     // show new content
}

/*!
  This member paints the cell borders and the text in each cell.
  This is where I implement a blue color for the column headings.
*/

void SQLTable::paintCell( QPainter* p, int row, int col )
{
  QFontMetrics fm( font() );
  
  int w = cellWidth( col );                   
  int h = cellHeight( row );                  
  int x2 = w - 1;
  int y2 = h - 1;

  p->drawLine( x2, 0, x2, y2 );              
  p->drawLine( 0, y2, x2, y2 );         
  

  if( row == 0 )
    p->setPen( blue );
  
  p->drawText( 5, 5, fm.width( contents[indexOf(row,col)] ), h, AlignLeft, contents[indexOf(row,col)] );
  
  p->setPen( black );  
}

/*!
  This member is a public member to set the number of rows that
  will be in the table.
*/

void SQLTable::setRows( int rows )
{
  rowNum = rows;
  contents->resize( rowNum * colNum );
  setNumRows( rows );
}

/*!
  This member is a public member to set the number of columns
  that will be in the table.
*/

void SQLTable::setCols( int cols )
{
  colNum = cols;
  contents->resize( rowNum * colNum );
  setNumCols( cols );
}

/*!
  This member is used by the setCell member.
*/

int SQLTable::indexOf( int row, int col ) const
{
  return (row * numCols()) + col;
}

/*!
  This member is used to clear all the data in the table.
*/

void SQLTable::clearTable()
{
  setRows(0);
  setCols(0);
  setCellWidth(0);
  setCellHeight(0);
}
